/*
	Planimeter - A Mathematical Instrument
	Copyright © 2007 Harry Whitfield

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2 of the License, or (at your
	option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

	Planimeter - version 1.0.3
	12 February, 2007
	Copyright © 2007 Harry Whitfield
	mailto:g6auc@arrl.net
*/

var deltaX =  1;	// adjustments for the mouse pointer
var deltaY =  0;

var pivotX = pivot.hOffset;
var pivotY = pivot.vOffset;

var elbowX = elbow.hOffset;
var elbowY = elbow.vOffset;

var blueLength  = 500;
var greenLength = 620;

var scaleFactor = blueLength*greenLength;

var area = 0;
var oldRotation = 0;	// in radians
var clockwise = 1;		// anti-clockwise = -1

var counter = [];
var counterSize = 8;
var over = Math.pow(10, counterSize);

var base = 'Resources/Images/Digits/';	// base for digit images

function makeCounter(counter, size, window, hOff, vOff, hRegP, vRegP, rot)
{
	function newImage(window, src, width, height, hOff, vOff, hRegP, vRegP, rot)
	{
		var img = new Image();
		img.window = window;
		img.src = src;
		img.width = width;
		img.height = height;
		img.hOffset = hOff;
		img.vOffset = vOff;
		img.hRegistrationPoint = hRegP;
		img.vRegistrationPoint = vRegP;
		img.rotation = rot;
		return img;
	}
	
	for (var i = 0; i < size; i += 1)
	{
		counter[i] = newImage(window, base + 'digit0.png', 14, 20, hOff, vOff, hRegP - 14*i, vRegP, rot);
		counter[i].tooltip = 'Negative values are displayed in tens-complement notation';
	}
}

function moveCounter(counter, hOff, vOff, rotation)
{
	for (var i = 0; i < counter.length; i += 1)
	{
		counter[i].hOffset = hOff;
		counter[i].vOffset = vOff;
		counter[i].rotation = rotation;
	}
}

function setCounter(num)
{
	num = num % over;
	if (num < 0) { num = over + num; }
	
	for (var i = counter.length - 1; i >= 0; i -= 1)
	{
		var digit = num % 10;
		counter[i].src = base + 'digit' + digit + '.png';
		num = (num - digit) / 10;
	}
}

function multiClick(obj)
{
	if (obj == areaText)
	{
		area = 0; areaText.data = "000000"; oldRotation = blueArm.rotation*Math.PI/180;
		setCounter(0);
	}
	else
	{
		blueArm.opacity =
		pivot.opacity =
		pivotText.opacity =
		greenArm.opacity =
		elbow.opacity =
		elbowText.opacity =
		target.opacity = 287 - blueArm.opacity;
		for (var i = 0; i < counter.length; i +=1) { counter[i].opacity = greenArm.opacity; }
	}
}

function mouseMv(img)
{
	var mvX = system.event.hOffset + deltaX;
	var mvY = system.event.vOffset + deltaY;
	
	if (mvX <= pivot.hOffset + 5) { mvX = pivot.hOffset + 5; }
	if (mvY <= pivot.vOffset + 5) { mvY = pivot.vOffset + 5; }

	var coords = intersection(pivotX, pivotY, blueLength,  mvX, mvY, greenLength);
	if (coords !== null)
	{
		elbowX = coords[2]; elbowY = coords[3];

		var theta = Math.atan2(mvY - elbowY, mvX - elbowX);
		var rotation = Math.atan2(elbowY - pivotY, elbowX - pivotX);
		var phi = rotation - oldRotation; oldRotation = rotation;
		area += scaleFactor*phi*Math.cos(rotation - theta);

		target.hOffset = dragSpot.hOffset = greenArm.hOffset = elbow.hOffset = elbowText.hOffset = elbowX;
		target.vOffset = dragSpot.vOffset = greenArm.vOffset = elbow.vOffset = elbowText.vOffset = elbowY;
		elbowText.vOffset += 5;

		target.rotation = dragSpot.rotation = greenArm.rotation = Math.round(theta*180/Math.PI);
		elbowText.data = normalized(clockwise * greenArm.rotation);
		
		blueArm.rotation = Math.round(rotation*180/Math.PI);
		pivotText.data = normalized(clockwise * blueArm.rotation);
		
		moveCounter(counter, greenArm.hOffset, greenArm.vOffset, greenArm.rotation);

		areaText.data = Math.round(clockwise * area);
		
		setCounter(Math.round(clockwise * area));
	}
}

function normalized(rotation)
{
	rotation = ((rotation % 360) + 360) % 360;
	if (rotation > 180) { rotation = rotation - 360; }
	return rotation;
}

function intersection(x1, y1, r1,  x2, y2, r2)	// of two circles
{
	if (y1 != y2)
	{
		var alpha = (x1 - x2) / (y2 - y1);
		var beta  = (r1*r1 - x1*x1 - y1*y1 - r2*r2 + x2*x2 + y2*y2) / (2*(y2 - y1));
		var a = alpha*alpha + 1;
		var b = 2*(alpha*(beta - y1) - x1);
		var c = (beta - y1)*(beta - y1) - r1*r1 + x1*x1;
		var d = b*b - 4*a*c; if (d < 0) { return null; }	// circles don't intersect
		var s = Math.sqrt(d);
		var xRes1 = (-b + s) / (2*a);
		var xRes2 = (-b - s) / (2*a);
		var yRes1 = alpha*xRes1 + beta;
		var yRes2 = alpha*xRes2 + beta;
	}
	else
	{
		xRes1 = (r1*r1 - x1*x1 - r2*r2 + x2*x2) / (2*(x2 - x1));
		xRes2 = xRes1;
		d = r1*r1 - (xRes1 - x1)*(xRes1 - x1); if (d < 0) { return null; }	// circles don't intersect
		s = Math.sqrt(d);
		yRes1 = y1 + s;
		yRes2 = y1 - s;
	}
	return [xRes1, yRes1, xRes2, yRes2];
}

function updatePrefs()
{
	blueLength  = Number(preferences.rotorLengthPref.value);
	greenLength = Number(preferences.slideLengthPref.value);
	
	blueArm.clipRect  = "0,0," + blueLength  + ",36";
	greenArm.clipRect = "0,0," + greenLength + ",36";
	
	dragSpot.hRegistrationPoint = target.hRegistrationPoint = 0 - greenLength;
	
	greenLength += 20;
	
	elbowText.hOffset = elbow.hOffset  = dragSpot.hOffset = target.hOffset = greenArm.hOffset  = blueArm.hOffset + 3*blueLength/5;
						elbow.vOffset  = dragSpot.vOffset = target.vOffset = greenArm.vOffset  = blueArm.vOffset + 4*blueLength/5;
	elbowText.vOffset = elbow.vOffset + 5;
	
	blueArm.rotation  =  53;
	target.rotation = dragSpot.rotation = greenArm.rotation = -10;
	
	moveCounter(counter, greenArm.hOffset, greenArm.vOffset, greenArm.rotation);

	area = 0; areaText.data = "000000"; oldRotation = blueArm.rotation*Math.PI/180;
	
	setCounter(0);
	
	scaleFactor = blueLength*greenLength;

	if (preferences.clockwisePref.value == "1") { clockwise = 1; } else { clockwise = -1; }
	
	elbowText.data = normalized(clockwise * greenArm.rotation);
	pivotText.data = normalized(clockwise * blueArm.rotation);

	blueArm.colorize  = preferences.rotorColorPref.value;
	target.colorize   =
	greenArm.colorize = preferences.slideColorPref.value;
	pivot.colorize    = 
	elbow.colorize    = 
	areaText.bgColor  = preferences.bkgdColorPref.value;
	pivotText.color   =
	elbowText.color   =
	areaText.color    = preferences.textColorPref.value;
}
